<?php

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\Cache\Exception;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Allocation {


    public  $allocationFactory;

    const ASSIGNER  = Entities\Allocation::TYPE_ASSIGNER;
    const REPORTER  = Entities\Allocation::TYPE_REPORTER;
    const FOLLOWER  = Entities\Allocation::TYPE_FOLLOWER;
    const SPONSOR   = Entities\Allocation::TYPE_SPONSOR;
    const RELEVANT = Entities\Allocation::RELEVANT;
    const NOT_RELEVANT = Entities\Allocation::NOT_RELEVANT;

    public $repositoryAllocation;
    /**
     * @var EntityManager
     */
    public $entityManager;
    /**
     * @var Utilities
     */
    public $environment;

    public function __construct(){
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();
        $this->allocationFactory = InstanceProvider::getFactory()->getAllocation();
        $this->repositoryAllocation = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation');
    }

    public function createAllocation($task, $resource, $type = null,$taskStatus = null, $isNotFlush = false, $isRelevant = self::NOT_RELEVANT){

        if(empty($taskStatus)){
            $taskStatus = Utilities::getTaskStatusNotInitialized();
        }
        if(empty($isRelevant)){
            $isRelevant = self::NOT_RELEVANT;
        }
        $entity = $this->allocationFactory->createAllocation($task, $resource, $type,$taskStatus, $isRelevant);
        $this->entityManager->persist($entity);
        if(!$isNotFlush) {
            $this->entityManager->flush();
        }
        return $entity;

    }
    public function updateAllocation(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $entity, $task, $resource, $type = null,$taskStatus = null, $isNotFlush = null, $isRelevant = self::NOT_RELEVANT){
        $entity = $this->allocationFactory->updateAllocation($entity, $task, $resource, $type,$taskStatus, $isRelevant);
        $this->entityManager->persist($entity);
        if(!$isNotFlush) {
            $this->entityManager->flush();
        }
        return $entity;

    }

    public function getAllocationByUserAndTask($resourceId, $taskId){
        $allocation = null;
        if(!empty($resourceId) && !empty($taskId)){
            $allocation = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation')
                        ->findOneBy(array("resource" => $resourceId, "task" => $taskId));
        }
        return $allocation;
    }
    public function setPlayDateTask($taskId,$resourceId,$initTask){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocation = $this->repositoryAllocation->findOneBy(array('task'=>$taskId,'resource'=> $resourceId));
        if($allocation && $initTask){
            $allocation->setPlayDate(\DateTime::createFromFormat('d/m/Y H:i:s',$initTask));
        }


    }
    public function updateTaskStatusAllocation($allocation, $taskStatus){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocation->setTaskStatus($taskStatus);
        $this->entityManager->persist($allocation);
        $this->entityManager->flush();
    }
    public function updateTaskStatusAllocationByTask($task, $taskStatus){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocations = $this->repositoryAllocation->findBy(array('task'=>$task));
        foreach($allocations as $allocation) {
            $this->updateTaskStatusAllocation($allocation, $taskStatus);
        }
    }

}
