import { InstanceService } from '@/services/InstanceService';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Http } from '@zeedhi/core';

export class DMBudgetIssueController {
	public callRelationship = '';

	public callType = 'I';

	public async beforeCancelGridBudgetItensIssue() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		await budgetItensGridInstance.datasource.get();
	}

	public validateNumberParcel(parcel: number) {
		let validate = true;
		if (parcel >= 100) {
			AlertServiceCustom.customErrorAlert('ERR_MANY_PLOTS');
			validate = false;
		} else if (parcel <= 0) {
			AlertServiceCustom.customErrorAlert('ERR_ZERO_NUM_PLOTS');
			validate = false;
		} else if (!parcel) {
			AlertServiceCustom.customErrorAlert('ERR_INVALID_NUM_PLOTS');
			validate = false;
		}
		return validate;
	}

	public setValuesAddParcel(parcel: any, widgetPlots: TekGrid) {
		const diff = parseFloat(parcel) - (widgetPlots.datasource.data.length) + 1;
		const valueTotal = Math.trunc(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS * 100) / 100;
		const value = Math.trunc((valueTotal / diff) * 100) / 100;
		const rest = valueTotal - ((value * 100 * diff) / 100);
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS = value;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
		return [value, rest];
	}

	public removeParcel(widgetPlots: TekGrid) {
		const row = widgetPlots.datasource.data.pop();
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS = ((parseFloat(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS) * 100) + parseFloat(row?.VALUE_PLOTS) * 100) / 100;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
	}

	public addParcel(widgetPlots: TekGrid, value: any, parcelLabel: string, parcel: number | null, rest: any) {
		const newRow = {
			VALUE_PLOTS: parseFloat(value),
			changed: true,
			ORDER_PLOTS: `${widgetPlots.datasource.data.length}° ${parcelLabel}`,
			ORDER_PLOTS_NUM: widgetPlots.datasource.data.length,
			DAYS_PAYMENT: parseInt(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].DAYS_PAYMENT, 10) + 30,
			BUDGET_ID: widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].BUDGET_ID,
		};
		if (parcel === widgetPlots.datasource.data.length) {
			newRow.VALUE_PLOTS += parseFloat(rest);
		}
		widgetPlots.datasource.data.push(newRow);
	}

	public async quantityPlotsChangeFormBudgetInstallmentsIssue() {
		debugger;
		const widgetPlots = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		const formBudgetInstallmentsIssue = InstanceService.getInstanceZdForm('formBudgetInstallmentsIssue');
		const parcel = parseInt(formBudgetInstallmentsIssue.value.QUANTITY_PLOTS, 10);
		let value = null;
		let rest = null;
		const parcelLabel = 'Parcelas';
		// widgetPlots.saveActionClick = _saveActionClick;
		if (this.validateNumberParcel(parcel)) {
			if (parcel === 1) {
				widgetPlots.datasource.data = [widgetPlots.datasource.data[0]];
				widgetPlots.datasource.data[0].VALUE_PLOTS = widgetPlots.datasource.data[0].PROPOSAL_VALUE;
			} else {
				if (parcel > widgetPlots.datasource.data.length) {
					const ret = this.setValuesAddParcel(parcel, widgetPlots);
					value = ret[0];
					rest = ret[1];
				}
				while (parcel !== widgetPlots.datasource.data.length) {
					if (parcel < widgetPlots.datasource.data.length) {
						this.removeParcel(widgetPlots);
					} else {
						this.addParcel(widgetPlots, value, parcelLabel, parcel, rest);
					}
				}
			}
			this.widgetPlotsUpdateGroups();
		}
	}

	public widgetPlotsUpdateGroups() {
		const widget = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		const groupedData = widget.groupedData.filter((row) => {
			let filter = false;
			if (row.group) {
				filter = row.group;
			}
			return filter;
		});
		let total = 0;
		widget.datasource.data.forEach((row) => {
			groupedData.push(row);
			total += row.VALUE_PLOTS;
		});
		widget.groupedData.forEach((row) => {
			if (row.groupFooter) {
				row.VALUE_PLOTS = total;
				groupedData.push(row);
			}
		});
		widget.groupedData = groupedData;
	}

	public async beforeSaveGridBudgetItensIssue(params: any) {
		params.event?.preventDefault();
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		await Http.post('/issueBudgetSave', {
			CALL_RELATIONSHIP: this.callRelationship,
			CALL_TYPE: this.callType,
			DATA: budgetItensGridInstance.groupedData,
		});
		await budgetItensGridInstance.reload();
		await budgetItensGridInstance.cancelEditedRows();
	}

	public async changeTabBudgetIssue(params: any) {
		this.callRelationship = params.component.events.callRelationship();
		try {
			const tabBudgetIssue = InstanceService.getInstanceTabs('tabBudgetIssue');
			const tab = tabBudgetIssue.tabs[tabBudgetIssue.activeTab];
			setTimeout(async () => {
				if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
					await tab.events.change(params);
				}
			}, 100);
		} catch (e) {
			setTimeout(async () => {
				await this.changeTabBudgetIssue(params);
			}, 1000);
		}
	}

	public async changeTabFrameBudgetInstallmentsIssue() {
		const gridBudgetInstallmentsIssue = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		await gridBudgetInstallmentsIssue.datasource.get();
	}

	public async changeTabFrameBudgetItensIssue() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		await budgetItensGridInstance.datasource.get();
	}

	public changeProposalQuantityGridBudgetItensIssue(params: any) {
		let proposalQuantity = params.component.value;
		if (!proposalQuantity) {
			proposalQuantity = 0;
		} else if (proposalQuantity < 0) {
			proposalQuantity *= -1;
		}
		const currentRow = params.row;
		currentRow.PROPOSAL_QUANTITY = proposalQuantity;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');

		if (currentRow.BUDGET_ITEM_ID === '999999XXXX') {
			if (proposalQuantity > 100) {
				currentRow.PROPOSAL_QUANTITY = 100;
			}
			const discountPercent = currentRow.PROPOSAL_QUANTITY;
			const indexSubTotal = widget.groupedData.length - 5;
			const subTotalProposalString = widget.groupedData[indexSubTotal].PROPOSAL_VALUE;
			const subTotalProposal = parseFloat(subTotalProposalString);
			const discount = Number(((subTotalProposal * discountPercent) / 100).toFixed(2)) * -1;
			currentRow.PROPOSAL_VALUE = Number(discount);
		} else {
			currentRow.PROPOSAL_VALUE = proposalQuantity * currentRow.ITEM_VALUE_TRUE;
		}
		this.updateGruopData(widget);
	}

	public updateGruopData(widget: TekGrid) {
		const indexTotal = widget.groupedData.length - 1;
		const indexSubTotalDescon = widget.groupedData.length - 2;
		const indexDescon = widget.groupedData.length - 3;
		const indexSubTotal = widget.groupedData.length - 5;
		let total = 0;
		widget.groupedData.forEach((row, index) => {
			if (!row.group) {
				if (index !== indexSubTotal
					&& index !== indexSubTotalDescon
					&& index !== indexTotal
				) {
					total += row.PROPOSAL_VALUE;
				} else if (index === indexSubTotalDescon) {
					row.PROPOSAL_VALUE = widget.groupedData[indexDescon].PROPOSAL_VALUE;
				} else {
					row.PROPOSAL_VALUE = total;
				}
			}
		});
		const discountValue = parseFloat(widget.groupedData[indexDescon].PROPOSAL_VALUE);
		const subTotalProposal = parseFloat(widget.groupedData[indexSubTotal].PROPOSAL_VALUE);
		let discount = Number(((discountValue * 100) / subTotalProposal).toFixed(2)) * -1;
		if (Number(discount) > 100) {
			discount = 100;
			widget.groupedData[indexDescon].PROPOSAL_VALUE = (-widget.groupedData[indexSubTotal].PROPOSAL_VALUE);
			widget.groupedData[indexTotal].PROPOSAL_VALUE = 0;
		}
		widget.groupedData[indexSubTotalDescon].PROPOSAL_VALUE = widget.groupedData[indexDescon].PROPOSAL_VALUE;
		widget.groupedData[indexDescon].PROPOSAL_QUANTITY = Number(discount);
	}
}
